@@ -9,6 +9,7 @@ import requests
 
 AUTH_URI = os.environ.get('AUTH_URI', 'localhost:5000/auth')
 AUTH_URL = os.environ.get('AUTH_URL', AUTH_URI)
+INFO_URL = os.environ.get('INFO_URL', 'localhost:5000/info')
 STICKY_AUTH_URL = os.environ.get('STICKY_AUTH_URL', AUTH_URL)
 
 USE_REDIS = os.environ.get('AUTH_USE_REDIS', "false") == "true"
@@ -25,9 +26,11 @@ if USE_REDIS:
         host=os.environ.get('REDISHOST', 'localhost'),
         port=int(os.environ.get('REDISPORT', 6379)))
 
+
 class AuthorizationError(Exception):
     pass
 
+
 def get_usernames(user_ids, token=None):
     if AUTH_DISABLED:
         return []
@@ -36,8 +39,9 @@ def get_usernames(user_ids, token=None):
         raise ValueError('missing token')
     if len(user_ids):
         users_request = requests.get(f"https://{AUTH_URL}/api/v1/username?id={','.join(map(str, user_ids))}",
-            headers={'authorization': 'Bearer ' + token},
-            timeout=5)
+                                     headers={
+                                         'authorization': 'Bearer ' + token},
+                                     timeout=5)
 
         if users_request.status_code in [401, 403]:
             raise AuthorizationError(users_request.text)
@@ -49,12 +53,15 @@ def get_usernames(user_ids, token=None):
     else:
         return []
 
+
 @cachetools.func.ttl_cache(maxsize=CACHE_MAXSIZE, ttl=CACHE_TTL)
 def user_cache_http(token):
-    user_request = requests.get(f"https://{AUTH_URL}/api/v1/user/cache", headers={'authorization': 'Bearer ' + token})
+    user_request = requests.get(
+        f"https://{AUTH_URL}/api/v1/user/cache", headers={'authorization': 'Bearer ' + token})
     if user_request.status_code == 200:
         return user_request.json()
 
+
 def get_user_cache(token):
     if USE_REDIS:
         cached_user_data = r.get("token_" + token)
@@ -63,6 +70,7 @@ def get_user_cache(token):
     else:
         return user_cache_http(token)
 
+
 @cachetools.func.ttl_cache(maxsize=CACHE_MAXSIZE, ttl=CACHE_TTL)
 def is_root_public(table_id, root_id, token):
     if root_id is None:
@@ -73,13 +81,15 @@ def is_root_public(table_id, root_id, token):
 
     url = f"https://{AUTH_URL}/api/v1/table/{table_id}/root/{root_id}/is_public"
 
-    req = requests.get(url, headers={'authorization': 'Bearer ' + token}, timeout=5)
+    req = requests.get(
+        url, headers={'authorization': 'Bearer ' + token}, timeout=5)
 
     if req.status_code == 200:
         return req.json()
     else:
         raise RuntimeError('is_root_public request failed')
 
+
 @cachetools.func.ttl_cache(maxsize=CACHE_MAXSIZE, ttl=CACHE_TTL)
 def table_has_public(table_id, token):
     if AUTH_DISABLED:
@@ -87,12 +97,26 @@ def table_has_public(table_id, token):
 
     url = f"https://{AUTH_URL}/api/v1/table/{table_id}/has_public"
 
-    req = requests.get(url, headers={'authorization': 'Bearer ' + token}, timeout=5)
+    req = requests.get(
+        url, headers={'authorization': 'Bearer ' + token}, timeout=5)
     if req.status_code == 200:
         return req.json()
     else:
         raise RuntimeError('has_public request failed')
 
+
+@cachetools.func.ttl_cache(maxsize=CACHE_MAXSIZE, ttl=CACHE_TTL)
+def dataset_from_table_id(service_namespace, table_id, token):
+    url = f"https://{INFO_URL}/api/v2/tablemapping/service/{service_namespace}/table/{table_id}/permission_group"
+    req = requests.get(
+        url, headers={'authorization': 'Bearer ' + token}, timeout=5)
+    if req.status_code == 200:
+        return req.json()
+    else:
+        raise RuntimeError(
+            f'failed to lookup dataset for service {service_namespace} & table_id: {table_id}: status code {req.status_code}. content: {req.content}')
+
+
 def auth_required(func=None, *, required_permission=None, public_table_key=None, public_node_key=None, service_token=None):
     def decorator(f):
         @wraps(f)
@@ -106,13 +130,16 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
                 return f(*args, **kwargs)
 
             flask.g.public_access_cache = None
+
             def lazy_check_public_access():
                 if flask.g.public_access_cache is None:
                     if service_token and required_permission is not 'edit':
                         if public_node_key is not None:
-                            flask.g.public_access_cache = is_root_public(kwargs.get(public_table_key), kwargs.get(public_node_key), service_token)
+                            flask.g.public_access_cache = is_root_public(kwargs.get(
+                                public_table_key), kwargs.get(public_node_key), service_token)
                         elif public_table_key is not None:
-                            flask.g.public_access_cache = table_has_public(kwargs.get(public_table_key), service_token)
+                            flask.g.public_access_cache = table_has_public(
+                                kwargs.get(public_table_key), service_token)
                         else:
                             flask.g.public_access_cache = False
                     else:
@@ -123,7 +150,8 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
             flask.g.public_access = lazy_check_public_access
 
             if AUTH_DISABLED:
-                flask.g.auth_user = {'id': 0, 'service_account': False, 'name': 'AUTH_DISABLED', 'email': 'AUTH_DISABLED@AUTH.DISABLED', 'admin': True, 'groups': [], 'permissions': {}}
+                flask.g.auth_user = {'id': 0, 'service_account': False, 'name': 'AUTH_DISABLED',
+                                     'email': 'AUTH_DISABLED@AUTH.DISABLED', 'admin': True, 'groups': [], 'permissions': {}}
                 flask.g.auth_token = 'AUTH_DISABLED'
                 return f(*args, **kwargs)
 
@@ -133,7 +161,8 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
             auth_header = flask.request.headers.get('authorization')
             xrw_header = flask.request.headers.get('X-Requested-With')
 
-            programmatic_access = xrw_header or auth_header or flask.request.environ.get('HTTP_ORIGIN')
+            programmatic_access = xrw_header or auth_header or flask.request.environ.get(
+                'HTTP_ORIGIN')
 
             AUTHORIZE_URI = 'https://' + STICKY_AUTH_URL + '/api/v1/authorize'
 
@@ -147,20 +176,25 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
             if auth_header:
                 if not auth_header.startswith('Bearer '):
                     resp = flask.Response("Invalid Request", 400)
-                    resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + '", error="invalid_request", error_description="Header must begin with \'Bearer\'"'
+                    resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + \
+                        '", error="invalid_request", error_description="Header must begin with \'Bearer\'"'
                     return resp
-                else: # auth header takes priority
-                    token = auth_header.split(' ')[1] # remove schema
+                else:  # auth header takes priority
+                    token = auth_header.split(' ')[1]  # remove schema
 
             if programmatic_access:
                 if not token and not flask.g.public_access():
                     resp = flask.Response("Unauthorized", 401)
-                    resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + '"'
+                    resp.headers['WWW-Authenticate'] = 'Bearer realm="' + \
+                        AUTHORIZE_URI + '"'
                     return resp
-            else: # direct browser access, or a non-browser request missing auth header (user error) TODO: check user agent to deliver 401 in this case
+            # direct browser access, or a non-browser request missing auth header (user error) TODO: check user agent to deliver 401 in this case
+            else:
                 if query_param_token:
-                    resp = flask.make_response(flask.redirect(furl(flask.request.url).remove([TOKEN_NAME, 'token']).url, code=302))
-                    resp.set_cookie(cookie_name, query_param_token, secure=True, httponly=True)
+                    resp = flask.make_response(flask.redirect(
+                        furl(flask.request.url).remove([TOKEN_NAME, 'token']).url, code=302))
+                    resp.set_cookie(cookie_name, query_param_token,
+                                    secure=True, httponly=True)
                     return resp
 
             cached_user_data = get_user_cache(token) if token else None
@@ -173,10 +207,12 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
                 return flask.redirect(AUTHORIZE_URI + '?redirect=' + quote(flask.request.url), code=302)
             elif not flask.g.public_access():
                 resp = flask.Response("Invalid/Expired Token", 401)
-                resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + '", error="invalid_token", error_description="Invalid/Expired Token"'
+                resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + \
+                    '", error="invalid_token", error_description="Invalid/Expired Token"'
                 return resp
             else:
-                flask.g.auth_user = {'id': 0, 'service_account': False, 'name': '', 'email': '', 'admin': False, 'groups': [], 'permissions': {}}
+                flask.g.auth_user = {'id': 0, 'service_account': False, 'name': '',
+                                     'email': '', 'admin': False, 'groups': [], 'permissions': {}}
                 flask.g.auth_token = None
                 return f(*args, **kwargs)
 
@@ -187,6 +223,7 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
     else:
         return decorator
 
+
 def auth_requires_admin(f):
     @wraps(f)
     @auth_required
@@ -202,7 +239,8 @@ def auth_requires_admin(f):
 
     return decorated_function
 
-def auth_requires_permission(required_permission, public_table_key=None, public_node_key=None, service_token=None, dataset=None):
+
+def auth_requires_permission(required_permission, public_table_key=None, public_node_key=None, service_token=None, dataset=None, table_arg='table_id'):
     def decorator(f):
         @wraps(f)
         @auth_required(required_permission=required_permission, public_table_key=public_table_key, public_node_key=public_node_key, service_token=service_token)
@@ -210,7 +248,7 @@ def auth_requires_permission(required_permission, public_table_key=None, public_
             if flask.request.method == 'OPTIONS':
                 return f(*args, **kwargs)
 
-            table_id = kwargs.get('table_id')
+            table_id = kwargs.get(table_arg)
 
             if table_id is None and dataset is None:
                 return flask.Response("Missing table_id", 400)
@@ -218,58 +256,39 @@ def auth_requires_permission(required_permission, public_table_key=None, public_
             local_dataset = dataset
 
             if local_dataset is None:
-                table_id_to_dataset = {
-                    "pinky100_sv16": "pinky100",
-                    "pinky100_neo1": "pinky100",
-                    "akhilesh-pinky100-0": "pinky100",
-                    "anp0": "pinky100",
-                    "minnie3_v0": "minnie65",
-                    "anm0": "minnie65",
-                    "minnie35_p3_v1": "minnie65",
-                    "fly_v26": "fafb_sandbox",
-                    "fly_training_v2": "fafb_training",
-                    "fly_v31": "fafb",
-                    "fly_arv0": "fafb",
-                    "pcgv2_fly_undo_redo": "fafb",
-                    "minnie65_public_v117": "microns_public",
-                    "minnie35_public_v0": "microns_public",
-                }
-
-                if table_id in table_id_to_dataset:
-                    local_dataset = table_id_to_dataset.get(table_id)
-                elif table_id.startswith("pinky100_rv") or \
-                        table_id.startswith("pinky100_arv") or \
-                        table_id.startswith("pinky_nf"):
-                    local_dataset = "pinky100"
-                elif table_id.startswith("minnie3_v"):
-                    local_dataset = "minnie65"
-                elif table_id.startswith("rsc_"):
-                    local_dataset = "RSC"
-                elif "fly" in table_id:
-                    local_dataset = "fafb"
-                else:
-                    return flask.Response("Unknown dataset", 400)
-
-            if local_dataset is not None:
-                has_permission = required_permission in flask.g.auth_user.get('permissions_v2', {}).get(local_dataset, [])
-
-                if not 'permissions_v2' in flask.g.auth_user: # backwards compatability
-                    required_level = ['none', 'view', 'edit'].index(required_permission)
-                    level_for_dataset = flask.g.auth_user.get('permissions', {}).get(local_dataset, 0)
-                    has_permission = level_for_dataset >= required_level
-
-                if AUTH_DISABLED or has_permission or flask.g.public_access(): # public_access won't be true for edit requests
-                    return f(*args, **kwargs)
-                else:
-                    resp = flask.Response("Missing permission: {0} for dataset {1}".format(required_permission, local_dataset), 403)
+                service_namespace = flask.current_app.config.get(
+                    'AUTH_SERVICE_NAMESPACE', 'datastack')
+                service_token_local = service_token if service_token else flask.current_app.config.get(
+                    'AUTH_TOKEN', "")
+                try:
+                    local_dataset = dataset_from_table_id(
+                        service_namespace, table_id, service_token_local)
+                except RuntimeError:
+                    resp = flask.Response("Invalid table_id for service", 400)
                     return resp
+
+            has_permission = required_permission in flask.g.auth_user.get(
+                'permissions_v2', {}).get(local_dataset, [])
+
+            if not 'permissions_v2' in flask.g.auth_user:  # backwards compatability
+                required_level = ['none', 'view', 'edit'].index(
+                    required_permission)
+                level_for_dataset = flask.g.auth_user.get(
+                    'permissions', {}).get(local_dataset, 0)
+                has_permission = level_for_dataset >= required_level
+
+            # public_access won't be true for edit requests
+            if AUTH_DISABLED or has_permission or flask.g.public_access():
+                return f(*args, **kwargs)
             else:
-                resp = flask.Response("Invalid table_id", 400)
+                resp = flask.Response("Missing permission: {0} for dataset {1}".format(
+                    required_permission, local_dataset), 403)
                 return resp
 
         return decorated_function
     return decorator
 
+
 def auth_requires_group(required_group):
     def decorator(f):
         @wraps(f)
@@ -279,7 +298,8 @@ def auth_requires_group(required_group):
                 return f(*args, **kwargs)
 
             if not AUTH_DISABLED and required_group not in flask.g.auth_user['groups']:
-                resp = flask.Response("Requires membership of group: {0}".format(required_group), 403)
+                resp = flask.Response(
+                    "Requires membership of group: {0}".format(required_group), 403)
                 return resp
 
             return f(*args, **kwargs)
